import { CallHandler, ExecutionContext, Injectable, NestInterceptor } from '@nestjs/common';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';

import { EntityType } from '../types/entity.type';
import { BaseEntity } from '../interfaces/base-entity.interface';
import { ResourceSerializer } from '../serializers/resource.serializer';
import { IMetaObject, IResponseObject, IJSONAPIObject } from '../interfaces/response';

export interface IEntitySerializerOptions {
  jsonApiEnabled?: boolean; 
  jsonApiMetaData?: IMetaObject;
}

@Injectable()
export class EntitySerializerInterceptor<T extends BaseEntity> implements NestInterceptor<T, IResponseObject<T>> {
  constructor(
    private type: EntityType<T>, 
    private options: IEntitySerializerOptions = { jsonApiEnabled: true }
  ) { }

  public intercept(context: ExecutionContext, next: CallHandler): Observable<IResponseObject<T>> {
    return next.handle().pipe(map(data => { 
      const response: IResponseObject<T> = {};
      
      /* Property: 'data' */
      if (Array.isArray(data)) {
        const serializer: ResourceSerializer<T> = new ResourceSerializer<T>(this.type);
        response.data = serializer.handleMany(data);
      } else {
        const serializer: ResourceSerializer<T> = new ResourceSerializer<T>(this.type);
        response.data = serializer.handleOne(data);
      }

      /* Property: 'jsonapi' */
      if (this.options.jsonApiEnabled) {
        const jsonapi: IJSONAPIObject = { version: '1.0', meta: this.options.jsonApiMetaData }
        response.jsonapi = jsonapi;
      }

      return response;
    }));
  }
}