import { EntityType } from '../types/entity.type';
import { BaseEntity } from '../interfaces/base-entity.interface';
import { IResourceIdentifierObject, IResourceObject } from '../interfaces/response';

export class ResourceSerializer<T extends BaseEntity> {
  constructor(private type: EntityType<T>) {}

  public handleOne(data: T): IResourceObject<T> | IResourceIdentifierObject<T> | null {
    const { id, ...attributes } = data;

    const options = Reflect.getMetadata('SERIALIZER_ENTITY', this.type);
    if (!options) {
      throw new Error(`Could not resolve entity type for class ${ this.type }. Are you missing an @Entity decorator?`);
    }
    
    return { id, type: options.name, attributes };
  }
  
  public handleMany(data: T[]): IResourceObject<T>[] | IResourceIdentifierObject<T>[] | [] {
    return [];
  }
}

// const relations = Reflect.getMetadata('SERIALIZER_RELATION', this.entity) || {};